/*
 * Copyright 2012 Stefan Haun, Andreas Nürnberger
 * 
 *      Data and Knowledge Engineering Group, 
 * 		Faculty of Computer Science,
 *		Otto-von-Guericke University,
 *		Magdeburg, Germany
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.ovgu.dke.mocca.tools.streaming;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URI;
import java.util.Collection;
import java.util.Collections;
import java.util.Properties;

import org.apache.commons.codec.binary.Base64OutputStream;

import de.ovgu.dke.mocca.api.MoccaException;
import de.ovgu.dke.mocca.api.command.Command;
import de.ovgu.dke.mocca.api.command.CommandHandler;
import de.ovgu.dke.mocca.api.context.Context;
import de.ovgu.dke.mocca.api.context.State.Lifecycle;
import de.ovgu.dke.mocca.util.ContextUtils;

public class ReceiveBinaryDataHandler implements CommandHandler {

	public static ReceiveBinaryDataHandler getInstance() {
		return new ReceiveBinaryDataHandler();
	}

	private ReceiveBinaryDataHandler() {
		// use instance control
		// see getInstance instead
	}

	@Override
	public Collection<URI> getAvailableCommands()  {
		return Collections.singletonList(StreamIdentifiers.CMD_BINARY_RECEIVE);
	}

	@Override
	public void handleCommand(Command cmd, Context ctx) throws MoccaException {
		// get the output stream
		Base64OutputStream dos = (Base64OutputStream) ctx
				.getAttribute(StreamIdentifiers.ATTR_DECODER);
		if (dos == null) {
			OutputStream fos = (OutputStream) ctx
					.getAttribute(StreamIdentifiers.ATTR_STREAM);

			// create a Base64 encoding input stream with no line separation
			dos = new Base64OutputStream(fos, false, 0, null);
			ctx.putAttribute(StreamIdentifiers.ATTR_DECODER, dos);
		}

		final String _chunk_size = cmd
				.getParameter(StreamIdentifiers.PARA_CHUNK_SIZE);
		Integer chunk_size = null;
		try {
			chunk_size = Integer.parseInt(_chunk_size);
		} catch (NumberFormatException nfe) {
			ContextUtils.sendError(
					ctx,
					"IllegalArgument",
					"Number format exception for chunk size: "
							+ nfe.getMessage());
			return;
		}

		final String chunk = cmd.getParameter(StreamIdentifiers.PARA_DATA);

		if (chunk_size != null && chunk != null) {
			final byte[] buf = new byte[chunk_size];

			for (int i = 0; i < chunk_size; i++) {
				buf[i] = (byte) chunk.charAt(i);
			}

			try {
				dos.write(buf);
			} catch (IOException e) {
				ContextUtils.sendError(
						ctx,
						"InternalError",
						"IO Exception while decoding stream data: "
								+ e.getMessage());
			}
		}
	}

	// TODO einbinden
	public void processStateUpdate(Command cmd, Context ctx)
			throws MoccaException {
		final String state = cmd.getParameter("state");

		if ("IDLE".equals(state) || "FAILED".equals(state)) {
			final Base64OutputStream dos = (Base64OutputStream) ctx
					.getAttribute(StreamIdentifiers.ATTR_DECODER);
			final OutputStream fos = (OutputStream) ctx
					.getAttribute(StreamIdentifiers.ATTR_STREAM);

			try {
				if (dos != null)
					dos.close();
				if (fos != null)
					fos.close();
				ContextUtils.sendProgress(ctx, Lifecycle.IDLE);
			} catch (IOException e) {
				ContextUtils.sendError(ctx, "InternalError",
						"Error closing streams: " + e.getMessage());
			}

			// TODO stream-attribute löschen
		}

	}

	public void receiveFile(Command cmd, Context ctx) throws MoccaException {
		// retrieve file name
		final String pdfName = cmd.getParameter("file");
		if (pdfName == null || pdfName.isEmpty()) {
			ContextUtils.sendError(ctx, "IllegalArgument",
					"File name 'file' is empty or has not been provided!");
			return;
		}

		// retrieve overwrite option:
		// file will not be overwritten iif overwrite is specified as false
		final String _overwrite = cmd.getParameter("overwrite");
		final boolean overwrite = _overwrite == null ? true : Boolean
				.parseBoolean(_overwrite);

		String targetDir = ""; // TODO
		// create the file output stream
		final File file = new File(targetDir + "/" + pdfName);

		// check if file exists and we may not overwrite
		if (file.exists() && !overwrite) {
			ContextUtils
					.sendError(
							ctx,
							"FileExists",
							"The target file already exists: "
									+ file.getAbsolutePath());
			return;
		}

		try {
			file.createNewFile();
			final FileOutputStream fos = new FileOutputStream(file);
			ctx.putAttribute(StreamIdentifiers.STREAM_PREFIX + "#file", file);
			ctx.putAttribute(StreamIdentifiers.ATTR_STREAM, fos);
		} catch (FileNotFoundException e) {
			ContextUtils.sendError(ctx, "InternalError",
					"Could not create target file: " + e.getMessage());
			return;
		} catch (IOException e) {
			ContextUtils.sendError(ctx, "InternalError",
					"Could not create target file: " + e.getMessage());
		}

		// response properties
		final Properties props = new Properties();
		props.put("remotename", file.getAbsolutePath());

		// start download
		ContextUtils.sendProgress(ctx, Lifecycle.RUNNING);

		// TODO ctx.registerCommandCallback("RECEIVE BINARY DATA", this);

		final Command send = ctx.getRuntime().createCommand(
				StreamIdentifiers.CMD_BINARY_SEND, props);
		ctx.sendCommand(send);

		// TODO XMPPProcessManager.getInstance().addProcessStatusListener(this,
		// process.getId());
	}
}