/*
 * Copyright 2012 Stefan Haun, Andreas Nürnberger
 * 
 *      Data and Knowledge Engineering Group, 
 * 		Faculty of Computer Science,
 *		Otto-von-Guericke University,
 *		Magdeburg, Germany
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.ovgu.dke.mocca.tools.streaming;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.util.Collection;
import java.util.Collections;
import java.util.Properties;

import org.apache.commons.codec.BinaryEncoder;
import org.apache.commons.codec.EncoderException;
import org.apache.commons.codec.binary.Base64;

import de.ovgu.dke.mocca.api.MoccaException;
import de.ovgu.dke.mocca.api.command.Command;
import de.ovgu.dke.mocca.api.command.CommandHandler;
import de.ovgu.dke.mocca.api.context.Context;
import de.ovgu.dke.mocca.api.context.State.Lifecycle;
import de.ovgu.dke.mocca.tools.streaming.chunking.Chunk;
import de.ovgu.dke.mocca.tools.streaming.chunking.ChunkIterator;
import de.ovgu.dke.mocca.util.ContextUtils;

public class SendBinaryDataHandler implements CommandHandler {
	private static BinaryEncoder encoder = new Base64(0);

	public static SendBinaryDataHandler getInstance() {
		return new SendBinaryDataHandler();
	}

	private SendBinaryDataHandler() {
		// use instance control
		// see getInstance instead
	}

	@Override
	public Collection<URI> getAvailableCommands() {
		return Collections.singletonList(StreamIdentifiers.CMD_BINARY_SEND);
	}

	@Override
	public void handleCommand(Command cmd, Context ctx) throws MoccaException {
		// get the data input stream
		final InputStream data_stream = (InputStream) ctx
				.getAttribute(StreamIdentifiers.ATTR_STREAM);
		if (data_stream == null) {
			ContextUtils
					.sendError(
							ctx,
							"IllegalStata",
							"Data stream could not be found. The context has not been initialized properly!");
			return;
		}

		Integer chunk_size;
		// get the chunk size
		try {
			chunk_size = ContextUtils.retrieveInteger(ctx, cmd,
					StreamIdentifiers.PARA_CHUNK_SIZE);
		} catch (NumberFormatException nfe) {
			// exception has already been handled, just return.
			return;
		}
		if (chunk_size == null)
			// we send 1 kb chunks by default
			chunk_size = 1 * 1024;

		// recalculate according to base64 encoding
		chunk_size = (int) Math.floor(chunk_size * 3 / 4) - 2;

		ContextUtils.sendProgress(ctx, Lifecycle.RUNNING);

		// create the chunk iterator
		ChunkIterator chunks = new ChunkIterator(data_stream, chunk_size);

		// common data properties
		final Properties props = new Properties();
		props.put(StreamIdentifiers.PARA_ENCODING, "base64");

		// send data while the process is running and there are chunks available
		// this may be interrupted externally by sending another state
		while (chunks.hasNext()
				&& ctx.getState().getLifecycle().equals(Lifecycle.RUNNING)) {
			// TODO exception handling
			final Chunk chunk = chunks.next();

			// send data
			props.put(StreamIdentifiers.PARA_CHUNK_NO, chunk.getNo());
			try {
				final byte[] encoded = encoder.encode(chunk.getData());
				props.put(StreamIdentifiers.PARA_CHUNK_SIZE, encoded.length);
				props.put(StreamIdentifiers.PARA_DATA, new String(encoded,
						"utf-8"));

				ctx.sendCommand(ctx.getRuntime().createCommand(
						StreamIdentifiers.CMD_BINARY_RECEIVE, props));
			} catch (UnsupportedEncodingException e) {
				ContextUtils.sendError(ctx, "InternalError",
						"Encoding utf-8 is not known! " + e.getMessage());
			} catch (EncoderException e) {
				ContextUtils.sendError(ctx, "InternalError",
						"Error encoding binary stream! " + e.getMessage());
			}

		} // while

		// cleanup
		ctx.putAttribute(StreamIdentifiers.ATTR_CHUNKS, null);

		// finished if still running
		if (ctx.getState().getLifecycle().equals(Lifecycle.RUNNING)) {
			ContextUtils.sendProgress(ctx, Lifecycle.IDLE);
		}
	}
}
