/*
 * Copyright 2012 Stefan Haun, Andreas Nürnberger
 * 
 *      Data and Knowledge Engineering Group, 
 * 		Faculty of Computer Science,
 *		Otto-von-Guericke University,
 *		Magdeburg, Germany
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.ovgu.dke.mocca.tools.streaming;

import java.net.URI;

/**
 * <p>
 * The type of the stream. Depending of this type, different meta information is
 * available.
 * </p>
 * 
 * @author Stefan Haun (stefan.haun@ovgu.de)
 * 
 */
public enum StreamType {
	/**
	 * Stream characteristic: no specified end or size and (optional)
	 * content-type information.
	 * <p>
	 * URI: @literal{http://dke.ovgu.de/mocca/tools/stream/type#stream}
	 * </p>
	 */
	STREAM("#stream"),
	/**
	 * Binary Large OBject (BLOB): stream with a specified length that avails
	 * transfer progress information.
	 * <p>
	 * URI: @literal{http://dke.ovgu.de/mocca/tools/stream/type#blob}
	 * </p>
	 */
	BLOB("#blob"),
	/**
	 * File: BLOB with additional file meta information: (original) path.
	 * <p>
	 * URI: @literal{http://dke.ovgu.de/mocca/tools/stream/type#file}
	 * </p>
	 */
	FILE("#file");

	/**
	 * Create a StreamType from a URI.
	 * 
	 * @param type
	 *            The type URI. Its base must be one of the valid type URIs
	 * @return a StreamType enum instance matching the URI
	 * @throws NullPointerException
	 *             if the type argument is @code{null}
	 * @throws IllegalArgumentException
	 *             if the URI is invalid.
	 */
	public static StreamType fromURI(URI type) {
		// The type must be a sub-URI of the ATTRIBUTE_TYPE URI.
		// We use a trick here: URI.relativize against the ATTRIBUTE_TYPE URI.
		// If the URIs match, we get the fragment part, otherwise a complete
		// URI.
		final String rel = StreamIdentifiers.PROPERTY_TYPE.relativize(type)
				.toString();

		// if the fragment is empty, the URI was incomplete
		if (rel.isEmpty())
			throw new IllegalArgumentException(
					"Invalid URI: Fragment is empty!");

		// If the base URI was correct, we have a fragment only, which starts
		// with a @literal{#}. Otherwise throw an exception.
		if (!rel.startsWith("#"))
			throw new IllegalArgumentException("Base URI must be "
					+ StreamIdentifiers.PROPERTY_TYPE + "!");

		return StreamType.valueOf(rel.substring(1).toUpperCase());
	}

	/**
	 * Create a StreamType from a String.
	 * 
	 * @param type
	 *            The type URI as String. Its base must be ATTRIUTE_TYPE and the
	 *            fragment one of @literal{#stream} , @literal{#blob} or
	 * @literal{#file .
	 * @return a StreamType enum instance matching the URI
	 * @throws NullPointerException
	 *             (optional) if the type argument is @code{null}
	 * @throws IllegalArgumentException
	 *             if the URI is unknown.
	 */
	public static StreamType fromString(String type) {
		for (final StreamType _t : StreamType.values())
			if (_t.toURI().toString().equals(type))
				return _t;

		throw new IllegalArgumentException("Invalid URI for stream type!");

		// TODO use this method for URI matching – will most likely be faster!
	}

	private final URI type;

	StreamType(String t) {
		this.type = URI.create(StreamIdentifiers.PROPERTY_TYPE + t);
	}

	/**
	 * Get the URI representation of this type
	 * 
	 * @return the URI for this type.
	 */
	public URI toURI() {
		return type;
	}

	/**
	 * Return the URI as String for this enum type.
	 */
	@Override
	public String toString() {
		return type.toString();
	}
}
