/*
 * Copyright 2012 Stefan Haun, Andreas Nürnberger
 * 
 *      Data and Knowledge Engineering Group, 
 * 		Faculty of Computer Science,
 *		Otto-von-Guericke University,
 *		Magdeburg, Germany
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.ovgu.dke.mocca.tools.streaming.chunking;

import java.util.Arrays;

import net.jcip.annotations.Immutable;

/**
 * A (byte array) chunk of data ready to be transferred. Each chunk has a number
 * which can be used to verify the stream integrity.
 * 
 * (Note: The base64 encode returns a byte stream containing the base64 String,
 * so we can handle the text transfer case as byte array, too. :) No need to
 * care for Strings or any channel encoding at this point. )
 * 
 * @author Stefan Haun (stefan.haun@ovgu.de)
 */
@Immutable
public class Chunk {
	/**
	 * Create a new chunk from the provided information.
	 * 
	 * @param data
	 *            the data to be contained in this chunk; may not be @code{null}
	 * @param no
	 *            the chunk number
	 * @return a Chunk instance.
	 * @throws NullPointerException
	 *             if the data argument is @code{null}.
	 **/
	public static Chunk newInstance(byte[] data, int no) {
		return new Chunk(data, data.length, no);
	}

	/**
	 * Create a new chunk from the provided information. Use this if the data
	 * buffer is not complete to avoid unnecessary copying of the data array.
	 * 
	 * @param data
	 *            the data to be contained in this chunk; may not be @code{null}
	 * @param length
	 *            number of bytes to be copied from the data
	 * @param no
	 *            the chunk number
	 * @return a Chunk instance.
	 * @throws NullPointerException
	 *             if the data argument is @code{null}.
	 * @throws NegativeArraySizeException
	 *             if the length argument is negative
	 **/
	public static Chunk newInstanceByPartialData(byte[] data, int length, int no) {
		return new Chunk(data, length, no);
	}

	private final byte[] data;
	private final int no;

	/**
	 * Internal constructor for data type T and a chunk number. Use
	 * 
	 * @code{newInstance to get a chunk instance.
	 * 
	 * @param data
	 *            the data to be contained in this chunk; may not be @code{null}
	 * @param length
	 *            number of bytes to be copied from the data
	 * @param no
	 *            the chunk number
	 * @throws NullPointerException
	 *             if the data argument is @code{null}.
	 * @throws NegativeArraySizeException
	 *             if the length argument is negative
	 */
	private Chunk(byte[] data, int length, int no) {
		if (data == null)
			throw new NullPointerException("The data argument may not be null!");

		// create a copy, otherwise the chunk is not immutable!
		this.data = Arrays.copyOf(data, length);
		this.no = no;
	}

	/**
	 * Get the data for this chunk.
	 * 
	 * @return the data.
	 */
	public byte[] getData() {
		return Arrays.copyOf(data, data.length);
	}

	/**
	 * Get the size of this chunk
	 * 
	 * @return the chunk size
	 */
	public int length() {
		return data.length;
	}

	/**
	 * Get the number of this chunk
	 * 
	 * @return chunk number
	 */
	public int getNo() {
		return no;
	}

	@Override
	public String toString() {
		return "Chunk " + no + " {length=" + data.length + ", data="
				+ Arrays.toString(data) + "}";
	}
}
