package de.ovgu.dke.mocca.tools.streaming.chunking;

import java.io.IOException;
import java.io.OutputStream;

import net.jcip.annotations.NotThreadSafe;

/**
 * Concatenate chunks into an output stream. The stream integrity is checked by
 * the chunk numbers, i.e. if the chunk number does not match the expectation,
 * pushing a chunk will fail.
 * 
 * This class is not thread-safe!
 * 
 * @author Stefan Haun (stefan.haun@ovgu.de)
 * 
 */
@NotThreadSafe
public class ChunkConcatenator {
	/**
	 * Get a Concatenator using the provided output stream as sink.
	 * 
	 * @param sink
	 *            The output stream where chunks are written.
	 * @returns a Concatenator instance using the provided output stream.
	 * @throws NullPointerException
	 *             if the sink argument is @code{null}.
	 */
	public static ChunkConcatenator fromStream(OutputStream sink) {
		return new ChunkConcatenator(sink);
	}

	private final OutputStream sink;

	private int nextNo;

	/**
	 * Create a Concatenator using the provided output stream as sink.
	 * 
	 * @param sink
	 *            The output stream where chunks are written.
	 * @throws NullPointerException
	 *             if the sink argument is @code{null}.
	 */
	private ChunkConcatenator(OutputStream sink) {
		if (sink == null)
			throw new NullPointerException("Sink argument must not be null!");

		this.sink = sink;

		this.nextNo = 0;
	}

	/**
	 * Get the sink output stream.
	 * 
	 * @return the output stream used as sink.
	 */
	public OutputStream getSink() {
		return sink;
	}

	/**
	 * Push a chunk to the output stream. The chunk number must match the
	 * expected next chunk number, otherwise the operation will fail with an
	 * exception.
	 * 
	 * @param chunk
	 *            The chunk to be pushed to the sink, must not be @code{null}.
	 * @throws ChunkException
	 *             if the chunk number does not match or writing to the output
	 *             stream fails.
	 * @throws NullPointerException
	 *             if the chunk argument is @code{null}.
	 */
	public void pushChunk(Chunk chunk) throws ChunkException {
		if (chunk == null)
			throw new NullPointerException("Chunk argument must not be null!");

		// check the chunk number
		if (chunk.getNo() != nextNo)
			throw new ChunkException("Chunk number mismatch: " + chunk.getNo()
					+ " (" + nextNo + " expected)");

		try {
			sink.write(chunk.getData());
		} catch (IOException e) {
			throw new ChunkException("Error writing chunk data: "
					+ e.getMessage(), e);
		}

		// next chunk number, if writing was successful
		nextNo++;
	}
}
