package de.ovgu.dke.mocca.tools.streaming.chunking;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.NoSuchElementException;

import net.jcip.annotations.NotThreadSafe;

/**
 * Iterate over an input stream and return the data as chunks.
 * 
 * If you want to catch exceptions during chunk creation (which involves stream
 * reading) you have to register an exception handler.
 * 
 * This class is not thead-safe!
 * 
 * @author Stefan Haun (stefan.haun@ovgu.de)
 * 
 */
// TODO Strategy: exact or fast (and smaller chunks?)
@NotThreadSafe
public class ChunkIterator implements Iterator<Chunk> {
	private final InputStream source;
	/**
	 * Target length for a chunk. Created chunks may actually be smaller.
	 */
	private final int length;

	private final byte[] buf;
	private Chunk next;
	private int no;

	private ChunkingExceptionHandler ex_handler;

	public ChunkIterator(InputStream source, int length) {
		if (source == null)
			throw new NullPointerException("Source stream may not be null!");
		this.source = source;
		this.length = length;

		// we use this buffer to create chunks
		this.buf = new byte[length];

		// the chunk number counter
		this.no = 0;
	}

	/**
	 * Set the Exception Handler to handle exception during chunk generation.
	 * 
	 * @param ex_handler
	 *            The exception handler for this iterator instance.
	 */
	public void setExceptionhandler(ChunkingExceptionHandler ex_handler) {
		this.ex_handler = ex_handler;
	}

	/**
	 * Get the target length for each chunk. Please note that a chunk may
	 * actually be smaller.
	 * 
	 * @return The target length for a chunk.
	 */
	public int getLength() {
		return length;
	}

	/**
	 * Get the chunk count.
	 * 
	 * @return the current status of the chunk number counter
	 */
	public int getNo() {
		return no;
	}

	@Override
	public boolean hasNext() {
		// provide the next chunk
		nextChunk();
		// true if there actually is a next chunk
		return this.next != null;
	}

	@Override
	public Chunk next() {
		// hasNext will get the next data chunk, if available
		if (!hasNext())
			throw new NoSuchElementException("No more chunks available!");

		// the current next chunk
		final Chunk _c = this.next;

		// reset the chunk store
		this.next = null;

		return _c;
	}

	/**
	 * @throws UnsupportedOperationException
	 *             removing is not supported in this iterator.
	 */
	@Override
	public void remove() {
		throw new UnsupportedOperationException(
				"Remove is not supported in the chunk iterator!");
	}

	/**
	 * Get the next chunk if not yet available. The chunk is stored in
	 * this.next, which contains null if no further chunk could be read.
	 */
	private void nextChunk() {
		if (this.next == null) {
			final int size;
			try {
				size = source.read(buf);
			} catch (IOException e) {
				if (ex_handler != null)
					ex_handler.handleChunkingException(e);
				this.next = null;
				return;
			}
			this.next = (size == -1) ? null : Chunk.newInstanceByPartialData(
					buf, size, no++);
		}
	}
}
