/*
 * Copyright 2012 Stefan Haun, Andreas Nürnberger
 * 
 *      Data and Knowledge Engineering Group, 
 * 		Faculty of Computer Science,
 *		Otto-von-Guericke University,
 *		Magdeburg, Germany
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.ovgu.dke.mocca.tools.streaming.transferobject;

import java.net.URI;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import de.ovgu.dke.mocca.tools.streaming.StreamType;

import net.jcip.annotations.Immutable;

/**
 * Abstract implementation for basic transfer object functions (type, meta
 * data). Implementing classes must add the data aspect and may add convenience
 * accessor method for specific meta information.
 * 
 * <p>
 * This part is immutable, however, implementing classes may not!
 * </p>
 * 
 * @author Stefan Haun (stefan.haun@ovgu.de)
 * 
 */
@Immutable
abstract class AbstractTransferObject implements TransferObject {
	private final StreamType type;
	private final Map<URI, String> metadata;

	private static final Map<URI, String> EMPTY_METADATA = Collections
			.unmodifiableMap(new HashMap<URI, String>());

	/**
	 * Create an abstract transfer object.
	 * 
	 * <p>
	 * Only available in package-scope.
	 * </p>
	 * 
	 * @param type
	 *            the transfer object type.
	 * @param metadata
	 *            the meta data as specified in {@link TransferObject}.
	 *            Additional meta data is allowed. May be @code{null} if no meta
	 *            data is specified.
	 * @throws NullPointerException
	 *             if type or meta data is @code{null}.
	 * @throws (optional) IllegalArgumentException if one of the arguments does
	 *         not meet the specification.
	 */
	AbstractTransferObject(final StreamType type,
			final Map<URI, String> metadata) {
		if (type == null)
			throw new NullPointerException("The type argument may not be null!");

		this.type = type;
		this.metadata = metadata == null ? EMPTY_METADATA : Collections
				.unmodifiableMap(new HashMap<URI, String>(metadata));
	}

	@Override
	public StreamType getType() {
		return type;
	}

	@Override
	public Map<URI, String> getMetaData() {
		return metadata;
	}

}
