/*
 * Copyright 2012 Stefan Haun, Andreas Nürnberger
 * 
 *      Data and Knowledge Engineering Group, 
 * 		Faculty of Computer Science,
 *		Otto-von-Guericke University,
 *		Magdeburg, Germany
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package de.ovgu.dke.mocca.tools.streaming.transferobject;

import java.io.InputStream;
import java.net.URI;
import java.util.Map;

import de.ovgu.dke.mocca.tools.streaming.StreamIdentifiers;
import de.ovgu.dke.mocca.tools.streaming.StreamType;

/**
 * A Transfer Object can be handed to the streaming API to be sent over a MOCCA
 * connection. The characteristic is represented by the type and influences the
 * available meta information.
 * 
 * <p>
 * Please note that transfer objects are most likely not immutable!
 * </p>
 * 
 * @author Stefan Haun (stefan.haun@ovgu.de)
 * 
 */
public interface TransferObject {
	static final String TYPE_PREFIX = StreamIdentifiers.STREAM_PREFIX + "/type";

	/**
	 * The content type of the stream data (as mime-type string).
	 */
	public static final URI ATTRIBUTE_CONTENT_TYPE = URI
			.create(StreamIdentifiers.STREAM_PREFIX + "/content-type");
	/**
	 * The length of a blob or file in bytes.
	 */
	public static final URI ATTRIBUTE_LENGTH = URI
			.create(StreamIdentifiers.STREAM_PREFIX + "/length");
	/**
	 * A path for the file. The interpretation may depend on the applicaton
	 * using this meta data.
	 */
	public static final URI ATTRIBUTE_PATH = URI
			.create(StreamIdentifiers.STREAM_PREFIX + "/path");

	/**
	 * Get the transfer object type.
	 * 
	 * @return one of the @code{TYPE_} identifiers.
	 */
	public StreamType getType();

	/**
	 * Get the transfer object meta data. Specific identifiers must match the
	 * 
	 * @code{ATTRIBUTE_ declarations.
	 * 
	 * @return meta data object, empty is none available, but never @code{null}!
	 */
	public Map<URI, String> getMetaData();

	/**
	 * Get the data stream.
	 * 
	 * @return Input Stream containing the data. The stream must match the type
	 *         and meta data!
	 */
	public InputStream getStream();
}
